<?php
/**
 * Created by PhpStorm.
 * User: crose
 * Date: 2/18/16
 * Time: 5:30 PM
 */

namespace qfq;

require_once(__DIR__ . '/../Constants.php');
require_once(__DIR__ . '/../exceptions/UserFormException.php');
require_once(__DIR__ . '/../helper/Support.php');

/**
 * Class Logger
 * @package qfq
 */
class Logger {

    /**
     * Append $msg to $filename.
     *
     * @param $msg
     * @param $filename
     *
     * @param string $mode
     * @throws UserFormException
     */
    public static function logMessage($msg, $filename, $mode = FILE_MODE_APPEND) {

        if ($filename == '') {
            return;
        }

        $filename = self::relativeToT3Dir($filename);

        if (!$handle = fopen($filename, $mode)) {

            // If open fails, maybe the directory does not exist. Create it:
            Support::mkDirParent($filename);

            // Try to open the file a second time.
            if (!$handle = fopen($filename, $mode)) {
                throw new UserFormException("Error - cannot open. File: " . $filename . " ( CWD: " . getcwd() . ") - " . error_get_last(), ERROR_IO_OPEN);
            }
        }

        if (fwrite($handle, $msg . PHP_EOL) === false) {
            throw new UserFormException("Error - cannot write. File: " . $filename . " ( CWD: " . getcwd() . ") - " . error_get_last(), ERROR_IO_WRITE);
        }

        fclose($handle);
    }

    /**
     * In case $filename is not absolute and if we're in the API directory: Check if we're in api - update relative filename
     *
     * @return string
     */
    private static function relativeToT3Dir($filename) {

        if (isset($filename[0]) && $filename[0] != '/') {
            if (strpos(getcwd(), '/qfq/qfq/api') !== false) {
                return ('../../../../../' . $filename);
            }
        }

        return $filename;
    }

    /**
     * @return string
     */
    public static function linePre() {
        $str = '=== [' . date('Y-m-d H:i:s');
        $str .= ' / ' . htmlentities(empty($_SERVER['REMOTE_ADDR']) ? '<no ip>' : $_SERVER['REMOTE_ADDR']);
        $str .= ' / ' . htmlentities(empty($_SERVER['HTTP_USER_AGENT']) ? '<no user agent>' : $_SERVER['HTTP_USER_AGENT']);
        $str .= '] ';

        return $str;
    }

    /**
     * @param array $fe
     *
     * @return string
     */
    public static function formatFormElementName(array $fe) {
        Support::setIfNotSet($fe, 'id');
        Support::setIfNotSet($fe, FE_NAME);
        Support::setIfNotSet($fe, FE_LABEL);

        return $fe['id'] . ' / ' . $fe[FE_NAME] . ' / ' . $fe[FE_LABEL];
    }

    /**
     * Logs a line to all of the given logfiles.
     * If $data is an array, it will be json encoded.
     *
     * @param array $form
     * @param $pre
     * @param $data
     * @throws UserFormException
     */
    public static function logFormLine(array $form, $pre, $data, $flagNewLineFirst = false) {

        $line = (is_array($data)) ? json_encode($data) : $data;
        $line = '[' . $pre . '] ' . $line;

        if ($flagNewLineFirst) {
            $line = PHP_EOL . $line;
        }

        foreach ([FORM_LOG_FILE_ALL, FORM_LOG_FILE_SESSION] as $filename) {
            if (!empty($form[$filename])) {
                Logger::logMessage($line, $form[$filename]);
            }
        }
    }
}