<?php
/**
 * Created by PhpStorm.
 * User: crose
 * Date: 3/31/16
 * Time: 10:06 PM
 */

namespace qfq;

require_once(__DIR__ . '/../../qfq/report/Link.php');
require_once(__DIR__ . '/../../qfq/store/Store.php');
require_once(__DIR__ . '/../../qfq/store/Sip.php');
require_once(__DIR__ . '/../../qfq/store/Session.php');
require_once(__DIR__ . '/../../qfq/exceptions/UserReportException.php');

use PHPUnit\Framework\TestCase;

//class LinkTest extends \PHPUnit_Framework_TestCase {

/**
 * Class LinkTest
 * @package qfq
 */
class LinkTest extends TestCase {

    /**
     * @var Sip
     */
    private $sip = null;

    /**
     * @var Store
     */
    private $store = null;

    /**
     * @expectedException \qfq\UserReportException
     */
    public function testUnknownTokenException1() {
        $link = new Link($this->sip, DB_INDEX_DEFAULT, true);

        $link->renderLink('x:hello world');
    }

    /**
     * @expectedException \qfq\UserReportException
     */
    public function testUnknownTokenException2() {
        $link = new Link($this->sip, DB_INDEX_DEFAULT, true);

        $link->renderLink('abc:hello world');
    }

    /**
     */
    public function testLinkUrlBasic() {
        $link = new Link($this->sip, DB_INDEX_DEFAULT, true);

        Store::setVar(TYPO3_PAGE_ID, 'firstPage', STORE_TYPO3);

        $result = $link->renderLink('');
        $this->assertEquals('', $result);

        $result = $link->renderLink('u:http://example.com');
        $this->assertEquals('<a href="http://example.com" >http://example.com</a>', $result);

        $result = $link->renderLink('u:http://example.com?id=100&t=2&param=hello');
        $this->assertEquals('<a href="http://example.com?id=100&t=2&param=hello" >http://example.com?id=100&t=2&param=hello</a>', $result);

        $result = $link->renderLink('u:example.com');
        $this->assertEquals('<a href="example.com" >example.com</a>', $result);

        $result = $link->renderLink('u:http://example.com|t:Hello world');
        $this->assertEquals('<a href="http://example.com" >Hello world</a>', $result);

        $result = $link->renderLink('u:http://example.com?id=100&t=2&param=hello|t:Hello world');
        $this->assertEquals('<a href="http://example.com?id=100&t=2&param=hello" >Hello world</a>', $result);
    }

    /**
     * @expectedException \qfq\UserReportException
     *
     */
    public function testLinkUrlBasicExceptionDouble() {
        $link = new Link($this->sip, DB_INDEX_DEFAULT, true);

        $link->renderLink('u:http://example.com|u:http://new.org');
    }

    /**
     * @expectedException \qfq\UserReportException
     *
     */
    public function testLinkUrlBasicExceptionEmpty1() {
        $link = new Link($this->sip, DB_INDEX_DEFAULT, true);

        $link->renderLink('u');
    }

    /**
     * @expectedException \qfq\UserReportException
     *
     */
    public function testLinkUrlBasicExceptionEmpty2() {
        $link = new Link($this->sip, DB_INDEX_DEFAULT, true);

        $link->renderLink('u:');
    }

    /**
     */
    public function testLinkPageBasic() {
        $link = new Link($this->sip, DB_INDEX_DEFAULT, true);
        Store::setVar(TYPO3_PAGE_ID, 'firstPage', STORE_TYPO3);

        $result = $link->renderLink('p');
        $this->assertEquals('<a href="?id=firstPage" >?id=firstPage</a>', $result);

        $result = $link->renderLink('p:');
        $this->assertEquals('<a href="?id=firstPage" >?id=firstPage</a>', $result);

        $result = $link->renderLink('p:secondPage');
        $this->assertEquals('<a href="?id=secondPage" >?id=secondPage</a>', $result);

        $result = $link->renderLink('p:id=secondPage');
        $this->assertEquals('<a href="?id=secondPage" >?id=secondPage</a>', $result);

        $result = $link->renderLink('p:id=secondPage&id=100&t=2&param=hello');
        $this->assertEquals('<a href="?id=secondPage&id=100&t=2&param=hello" >?id=secondPage&id=100&t=2&param=hello</a>', $result);

        $result = $link->renderLink('p:secondPage|t:Hello world');
        $this->assertEquals('<a href="?id=secondPage" >Hello world</a>', $result);
    }

    /**
     */
    public function testLinkMailBasic() {
        $link = new Link($this->sip, DB_INDEX_DEFAULT, true);

        $result = $link->renderLink('m:john@doe.com');
        $this->assertEquals('<a href="mailto:john@doe.com" >mailto:john@doe.com</a>', $result);

        $result = $link->renderLink('m:john@doe.com|t:John Doe');
        $this->assertEquals('<a href="mailto:john@doe.com" >John Doe</a>', $result);
    }

    /**
     * @expectedException \qfq\UserReportException
     *
     */
    public function testLinkMailBasicExceptionMissing1() {
        $link = new Link($this->sip, DB_INDEX_DEFAULT, true);

        $link->renderLink('m');
    }

    /**
     * @expectedException \qfq\UserReportException
     *
     */
    public function testLinkMailBasicExceptionMissing2() {
        $link = new Link($this->sip, DB_INDEX_DEFAULT, true);

        $link->renderLink('m:');
    }

    /**
     */
    public function testMailEncryption() {
        $link = new Link($this->sip, DB_INDEX_DEFAULT, true);

        //TODO: aktivieren sobald encrypted Mails implemented.
//        $result = $link->renderLink('m:john@doe.com|e');
//        $this->assertEquals('<a href="mailto:john@doe.com" >mailto:john@doe.com</a>', $result);
//
//        $result = $link->renderLink('m:john@doe.com|e:');
//        $this->assertEquals('<a href="mailto:john@doe.com" >mailto:john@doe.com</a>', $result);

        $result = $link->renderLink('m:john@doe.com|e:0');
        $this->assertEquals('<a href="mailto:john@doe.com" >mailto:john@doe.com</a>', $result);

//        $result = $link->renderLink('m:john@doe.com|e:1');
//        $this->assertEquals('<a href="mailto:john@doe.com" >mailto:john@doe.com</a>', $result);
    }

    /**
     * @throws CodeException
     * @throws UserFormException
     * @throws UserReportException
     */
    public function testRenderModeUrl() {
        $link = new Link($this->sip, DB_INDEX_DEFAULT, true);

        Store::setVar(TYPO3_PAGE_ID, 'firstPage', STORE_TYPO3);

        $result = $link->renderLink('u:http://example.com');
        $this->assertEquals('<a href="http://example.com" >http://example.com</a>', $result);

        $result = $link->renderLink('t:Example');
        $this->assertEquals('', $result);

        $result = $link->renderLink('u|t:Example');
        $this->assertEquals('', $result);

        $result = $link->renderLink('u:http://example.com|t:Example');
        $this->assertEquals('<a href="http://example.com" >Example</a>', $result);

        $result = $link->renderLink('u:http://example.com|r:0');
        $this->assertEquals('<a href="http://example.com" >http://example.com</a>', $result);

        $result = $link->renderLink('t:Example|r:0');
        $this->assertEquals('', $result);

        $result = $link->renderLink('u|t:Example|r:0');
        $this->assertEquals('', $result);

        $result = $link->renderLink('u:http://example.com|t:Example|r:0');
        $this->assertEquals('<a href="http://example.com" >Example</a>', $result);

        // r: 1
        $result = $link->renderLink('u:http://example.com|r:1');
        $this->assertEquals('<a href="http://example.com" >http://example.com</a>', $result);

        $result = $link->renderLink('t:Example|r:1');
        $this->assertEquals('Example', $result);

        $result = $link->renderLink('u|t:Example|r:1');
        $this->assertEquals('Example', $result);

        $result = $link->renderLink('u:http://example.com|t:Example|r:1');
        $this->assertEquals('<a href="http://example.com" >Example</a>', $result);

        // r: 2
        $result = $link->renderLink('u:http://example.com|r:2');
        $this->assertEquals('', $result);

        $result = $link->renderLink('t:Example|r:2');
        $this->assertEquals('', $result);

        $result = $link->renderLink('u|t:Example|r:2');
        $this->assertEquals('', $result);

        $result = $link->renderLink('u:http://example.com|t:Example|r:2');
        $this->assertEquals('<a href="http://example.com" >Example</a>', $result);

        // r: 3
        $result = $link->renderLink('u:http://example.com|r:3');
//        $this->assertEquals('<span >http://example.com</span>', $result);
        $this->assertEquals('http://example.com', $result);

        $result = $link->renderLink('t:Example|r:3');
        $this->assertEquals('Example', $result);

        $result = $link->renderLink('u|t:Example|r:3');
        $this->assertEquals('Example', $result);

        $result = $link->renderLink('u:http://example.com|t:Example|r:3');
//        $this->assertEquals('<span >Example</span>', $result);
        $this->assertEquals('Example', $result);

        // r: 4
        $result = $link->renderLink('u:http://example.com|r:4');
        $this->assertEquals('http://example.com', $result);

        $result = $link->renderLink('t:Example|r:4');
        $this->assertEquals('Example', $result);

        $result = $link->renderLink('u|t:Example|r:4');
        $this->assertEquals('Example', $result);

        $result = $link->renderLink('u:http://example.com|t:Example|r:4');
//        $this->assertEquals('<span >http://example.com</span>', $result);
        $this->assertEquals('http://example.com', $result);

        // r: 5
        $result = $link->renderLink('u:http://example.com|r:5');
        $this->assertEquals('', $result);

        $result = $link->renderLink('t:Example|r:5');
        $this->assertEquals('', $result);

        $result = $link->renderLink('u|t:Example|r:5');
        $this->assertEquals('', $result);

        $result = $link->renderLink('u:http://example.com|t:Example|r:5');
        $this->assertEquals('', $result);
    }

    /**
     * @throws CodeException
     * @throws UserFormException
     * @throws UserReportException
     */
    public function testRenderModePage() {
        $link = new Link($this->sip, DB_INDEX_DEFAULT, true);
        Store::setVar(TYPO3_PAGE_ID, 'firstPage', STORE_TYPO3);

        // r: default (0)
        $result = $link->renderLink('p');
        $this->assertEquals('<a href="?id=firstPage" >?id=firstPage</a>', $result);

        $result = $link->renderLink('p:secondPage');
        $this->assertEquals('<a href="?id=secondPage" >?id=secondPage</a>', $result);

        $result = $link->renderLink('t:Example');
        $this->assertEquals('', $result);

        $result = $link->renderLink('p:secondPage|t:Example');
        $this->assertEquals('<a href="?id=secondPage" >Example</a>', $result);

        // r: 0
        $result = $link->renderLink('p|r:0');
        $this->assertEquals('<a href="?id=firstPage" >?id=firstPage</a>', $result);

        $result = $link->renderLink('p:secondPage|r:0');
        $this->assertEquals('<a href="?id=secondPage" >?id=secondPage</a>', $result);

        $result = $link->renderLink('t:Example|r:0');
        $this->assertEquals('', $result);

        $result = $link->renderLink('p:secondPage|t:Example|r:0');
        $this->assertEquals('<a href="?id=secondPage" >Example</a>', $result);

        // r: 1
        $result = $link->renderLink('p|r:1');
        $this->assertEquals('<a href="?id=firstPage" >?id=firstPage</a>', $result);

        $result = $link->renderLink('p:secondPage|r:1');
        $this->assertEquals('<a href="?id=secondPage" >?id=secondPage</a>', $result);

        $result = $link->renderLink('t:Example|r:1');
//        $this->assertEquals('<span >Example</span>', $result);
        $this->assertEquals('Example', $result);

        $result = $link->renderLink('p:secondPage|t:Example|r:1');
        $this->assertEquals('<a href="?id=secondPage" >Example</a>', $result);

        // r: 2
        $result = $link->renderLink('p|r:2');
        $this->assertEquals('', $result);

        $result = $link->renderLink('p:secondPage|r:2');
        $this->assertEquals('', $result);

        $result = $link->renderLink('t:Example|r:2');
        $this->assertEquals('', $result);

        $result = $link->renderLink('p:secondPage|t:Example|r:2');
        $this->assertEquals('<a href="?id=secondPage" >Example</a>', $result);

        // r: 3
        $result = $link->renderLink('p|r:3');
//        $this->assertEquals('<span >?id=firstPage</span>', $result);
        $this->assertEquals('?id=firstPage', $result);

        $result = $link->renderLink('p:secondPage|r:3');
//        $this->assertEquals('<span >?id=secondPage</span>', $result);
        $this->assertEquals('?id=secondPage', $result);

        $result = $link->renderLink('t:Example|r:3');
//        $this->assertEquals('<span >Example</span>', $result);
        $this->assertEquals('Example', $result);

        $result = $link->renderLink('p:secondPage|t:Example|r:3');
//        $this->assertEquals('<span >Example</span>', $result);
        $this->assertEquals('Example', $result);

        // r: 4
        $result = $link->renderLink('p|r:4');
//        $this->assertEquals('<span >?id=firstPage</span>', $result);
        $this->assertEquals('?id=firstPage', $result);

        $result = $link->renderLink('p:secondPage|r:4');
//        $this->assertEquals('<span >?id=secondPage</span>', $result);
        $this->assertEquals('?id=secondPage', $result);

        $result = $link->renderLink('t:Example|r:4');
//        $this->assertEquals('<span >Example</span>', $result);
        $this->assertEquals('Example', $result);

        $result = $link->renderLink('p:secondPage|t:Example|r:4');
//        $this->assertEquals('<span >?id=secondPage</span>', $result);
        $this->assertEquals('?id=secondPage', $result);

        // r: 5
        $result = $link->renderLink('p|r:5');
        $this->assertEquals('', $result);

        $result = $link->renderLink('p:secondPage|r:5');
        $this->assertEquals('', $result);

        $result = $link->renderLink('t:Example|r:5');
        $this->assertEquals('', $result);

        $result = $link->renderLink('p:secondPage|t:Example|r:5');
        $this->assertEquals('', $result);
    }

    /**
     * @throws UserReportException
     */
    public function testRenderModeMail() {
        $link = new Link($this->sip, DB_INDEX_DEFAULT, true);

        // r: default (0)
        $result = $link->renderLink('m:john@doe.com');
        $this->assertEquals('<a href="mailto:john@doe.com" >mailto:john@doe.com</a>', $result);

        $result = $link->renderLink('t:Example');
        $this->assertEquals('', $result);

        $result = $link->renderLink('m:john@doe.com|t:Example');
        $this->assertEquals('<a href="mailto:john@doe.com" >Example</a>', $result);

        // r: 0
        $result = $link->renderLink('m:john@doe.com|r:0');
        $this->assertEquals('<a href="mailto:john@doe.com" >mailto:john@doe.com</a>', $result);

        $result = $link->renderLink('t:Example|r:0');
        $this->assertEquals('', $result);

        $result = $link->renderLink('m:john@doe.com|t:Example|r:0');
        $this->assertEquals('<a href="mailto:john@doe.com" >Example</a>', $result);

        // r: 1
        $result = $link->renderLink('m:john@doe.com|r:1');
        $this->assertEquals('<a href="mailto:john@doe.com" >mailto:john@doe.com</a>', $result);

        $result = $link->renderLink('t:Example|r:1');
//        $this->assertEquals('<span >Example</span>', $result);
        $this->assertEquals('Example', $result);

        $result = $link->renderLink('m:john@doe.com|t:Example|r:1');
        $this->assertEquals('<a href="mailto:john@doe.com" >Example</a>', $result);

        // r: 2
        $result = $link->renderLink('m:john@doe.com|r:2');
        $this->assertEquals('', $result);

        $result = $link->renderLink('t:Example|r:2');
        $this->assertEquals('', $result);

        $result = $link->renderLink('m:john@doe.com|t:Example|r:2');
        $this->assertEquals('<a href="mailto:john@doe.com" >Example</a>', $result);

        // r: 3
        $result = $link->renderLink('m:john@doe.com|r:3');
//        $this->assertEquals('<span >mailto:john@doe.com</span>', $result);
        $this->assertEquals('mailto:john@doe.com', $result);

        $result = $link->renderLink('t:Example|r:3');
//        $this->assertEquals('<span >Example</span>', $result);
        $this->assertEquals('Example', $result);

        $result = $link->renderLink('m:john@doe.com|t:Example|r:3');
//        $this->assertEquals('<span >Example</span>', $result);
        $this->assertEquals('Example', $result);

        // r: 4
        $result = $link->renderLink('m:john@doe.com|r:4');
//        $this->assertEquals('<span >mailto:john@doe.com</span>', $result);
        $this->assertEquals('mailto:john@doe.com', $result);

        $result = $link->renderLink('t:Example|r:4');
//        $this->assertEquals('<span >Example</span>', $result);
        $this->assertEquals('Example', $result);

        $result = $link->renderLink('m:john@doe.com|t:Example|r:4');
//        $this->assertEquals('<span >mailto:john@doe.com</span>', $result);
        $this->assertEquals('mailto:john@doe.com', $result);

        // r: 5
        $result = $link->renderLink('m:john@doe.com|r:5');
        $this->assertEquals('', $result);

        $result = $link->renderLink('t:Example|r:5');
        $this->assertEquals('', $result);

        $result = $link->renderLink('m:john@doe.com|t:Example|r:5');
        $this->assertEquals('', $result);
    }

    /**
     * @throws UserReportException
     */
    public function testRenderModeUrlPicture() {
        $link = new Link($this->sip, DB_INDEX_DEFAULT, true);

        // r: default (0)
        $result = $link->renderLink('u:http://example.com|P:picture.gif');
        $this->assertEquals('<a href="http://example.com" ><img alt="picture.gif" src="picture.gif" title="picture.gif" ></a>', $result);

        $result = $link->renderLink('t:Example|P:picture.gif');
        $this->assertEquals('', $result);

        $result = $link->renderLink('u:http://example.com|t:Example|P:picture.gif');
        $this->assertEquals('<a href="http://example.com" ><img alt="picture.gif" src="picture.gif" title="picture.gif" > Example</a>', $result);

        // r: 0
        $result = $link->renderLink('u:http://example.com|r:0|P:picture.gif');
        $this->assertEquals('<a href="http://example.com" ><img alt="picture.gif" src="picture.gif" title="picture.gif" ></a>', $result);

        $result = $link->renderLink('t:Example|r:0|P:picture.gif');
        $this->assertEquals('', $result);

        $result = $link->renderLink('u:http://example.com|t:Example|r:0|P:picture.gif');
        $this->assertEquals('<a href="http://example.com" ><img alt="picture.gif" src="picture.gif" title="picture.gif" > Example</a>', $result);

        // r: 1
        $result = $link->renderLink('u:http://example.com|r:1|P:picture.gif');
        $this->assertEquals('<a href="http://example.com" ><img alt="picture.gif" src="picture.gif" title="picture.gif" ></a>', $result);

        $result = $link->renderLink('t:Example|r:1|P:picture.gif');
//        $this->assertEquals('<span ><img alt="picture.gif" src="picture.gif" title="picture.gif" > Example</span>', $result);
        $this->assertEquals('<img alt="picture.gif" src="picture.gif" title="picture.gif" > Example', $result);

        $result = $link->renderLink('u:http://example.com|t:Example|r:1|P:picture.gif');
        $this->assertEquals('<a href="http://example.com" ><img alt="picture.gif" src="picture.gif" title="picture.gif" > Example</a>', $result);

        // r: 2
        //TODO: no link if text is empty - image is linked here: this is not what the user expects.
        $result = $link->renderLink('u:http://example.com|r:2|P:picture.gif');
        $this->assertEquals('<a href="http://example.com" ><img alt="picture.gif" src="picture.gif" title="picture.gif" ></a>', $result);

        $result = $link->renderLink('t:Example|r:2|P:picture.gif');
        $this->assertEquals('', $result);

        $result = $link->renderLink('u:http://example.com|t:Example|r:2|P:picture.gif');
        $this->assertEquals('<a href="http://example.com" ><img alt="picture.gif" src="picture.gif" title="picture.gif" > Example</a>', $result);

        // r: 3:
        $result = $link->renderLink('u:http://example.com|r:3|P:picture.gif');
//        $this->assertEquals('<span ><img alt="picture.gif" src="picture.gif" title="picture.gif" > </span>', $result);
        $this->assertEquals('<img alt="picture.gif" src="picture.gif" title="picture.gif" >', $result);

        $result = $link->renderLink('t:Example|r:3|P:picture.gif');
//        $this->assertEquals('<span ><img alt="picture.gif" src="picture.gif" title="picture.gif" > Example</span>', $result);
        $this->assertEquals('<img alt="picture.gif" src="picture.gif" title="picture.gif" > Example', $result);

        $result = $link->renderLink('u:http://example.com|t:Example|r:3|P:picture.gif');
//        $this->assertEquals('<span ><img alt="picture.gif" src="picture.gif" title="picture.gif" > Example</span>', $result);
        $this->assertEquals('<img alt="picture.gif" src="picture.gif" title="picture.gif" > Example', $result);

        // r: 4
        $result = $link->renderLink('u:http://example.com|r:4|P:picture.gif');
//        $this->assertEquals('<span >http://example.com</span>', $result);
        $this->assertEquals('http://example.com', $result);

        $result = $link->renderLink('t:Example|r:4|P:picture.gif');
//        $this->assertEquals('<span ><img alt="picture.gif" src="picture.gif" title="picture.gif" > Example</span>', $result);
        $this->assertEquals('<img alt="picture.gif" src="picture.gif" title="picture.gif" > Example', $result);

        $result = $link->renderLink('u:http://example.com|t:Example|r:4|P:picture.gif');
//        $this->assertEquals('<span >http://example.com</span>', $result);
        $this->assertEquals('http://example.com', $result);

        // r: 5
        $result = $link->renderLink('u:http://example.com|r:5|P:picture.gif');
        $this->assertEquals('', $result);

        $result = $link->renderLink('t:Example|r:5|P:picture.gif');
        $this->assertEquals('', $result);

        $result = $link->renderLink('u:http://example.com|t:Example|r:5|P:picture.gif');
        $this->assertEquals('', $result);
    }

    /**
     * @throws UserReportException
     */
    public function testRenderModeUrlButton() {
        $link = new Link($this->sip, DB_INDEX_DEFAULT, true);

        // r: default (0)
        $result = $link->renderLink('u:http://example.com|B:yellow');
        $this->assertEquals('<a href="http://example.com" ><img alt="Bullet yellow" src="typo3conf/ext/qfq/Resources/Public/icons/bullet-yellow.gif" title="yellow" ></a>', $result);

        $result = $link->renderLink('t:Example|B:yellow');
        $this->assertEquals('', $result);

        $result = $link->renderLink('u:http://example.com|t:Example|B:yellow');
        $this->assertEquals('<a href="http://example.com" ><img alt="Bullet yellow" src="typo3conf/ext/qfq/Resources/Public/icons/bullet-yellow.gif" title="yellow" > Example</a>', $result);

        // r: 0
        $result = $link->renderLink('u:http://example.com|r:0|B:yellow');
        $this->assertEquals('<a href="http://example.com" ><img alt="Bullet yellow" src="typo3conf/ext/qfq/Resources/Public/icons/bullet-yellow.gif" title="yellow" ></a>', $result);

        $result = $link->renderLink('t:Example|r:0|B:yellow');
        $this->assertEquals('', $result);

        $result = $link->renderLink('u:http://example.com|t:Example|r:0|B:yellow');
        $this->assertEquals('<a href="http://example.com" ><img alt="Bullet yellow" src="typo3conf/ext/qfq/Resources/Public/icons/bullet-yellow.gif" title="yellow" > Example</a>', $result);

        // r: 1
        $result = $link->renderLink('u:http://example.com|r:1|B:yellow');
        $this->assertEquals('<a href="http://example.com" ><img alt="Bullet yellow" src="typo3conf/ext/qfq/Resources/Public/icons/bullet-yellow.gif" title="yellow" ></a>', $result);

        $result = $link->renderLink('t:Example|r:1|B:yellow');
        $this->assertEquals('<img alt="Bullet yellow" src="typo3conf/ext/qfq/Resources/Public/icons/bullet-yellow.gif" title="yellow" > Example', $result);

        $result = $link->renderLink('u:http://example.com|t:Example|r:1|B:yellow');
        $this->assertEquals('<a href="http://example.com" ><img alt="Bullet yellow" src="typo3conf/ext/qfq/Resources/Public/icons/bullet-yellow.gif" title="yellow" > Example</a>', $result);

        // r: 2
        //TODO: no link if text is empty - image is linked here: this is not what the user expects.
        $result = $link->renderLink('u:http://example.com|r:2|B:yellow');
        $this->assertEquals('<a href="http://example.com" ><img alt="Bullet yellow" src="typo3conf/ext/qfq/Resources/Public/icons/bullet-yellow.gif" title="yellow" ></a>', $result);

        $result = $link->renderLink('t:Example|r:2|B:yellow');
        $this->assertEquals('', $result);

        $result = $link->renderLink('u:http://example.com|t:Example|r:2|B:yellow');
        $this->assertEquals('<a href="http://example.com" ><img alt="Bullet yellow" src="typo3conf/ext/qfq/Resources/Public/icons/bullet-yellow.gif" title="yellow" > Example</a>', $result);

        // r: 3:
        $result = $link->renderLink('u:http://example.com|r:3|B:yellow');
        $this->assertEquals('<img alt="Bullet yellow" src="typo3conf/ext/qfq/Resources/Public/icons/bullet-yellow.gif" title="yellow" >', $result);

        $result = $link->renderLink('t:Example|r:3|B:yellow');
//        $this->assertEquals('<span ><img alt="picture.gif" src="picture.gif" title="picture.gif" > Example</span>', $result);
        $this->assertEquals('<img alt="Bullet yellow" src="typo3conf/ext/qfq/Resources/Public/icons/bullet-yellow.gif" title="yellow" > Example', $result);

        $result = $link->renderLink('u:http://example.com|t:Example|r:3|B:yellow');
//        $this->assertEquals('<span ><img alt="picture.gif" src="picture.gif" title="picture.gif" > Example</span>', $result);
        $this->assertEquals('<img alt="Bullet yellow" src="typo3conf/ext/qfq/Resources/Public/icons/bullet-yellow.gif" title="yellow" > Example', $result);

        // r: 4
        $result = $link->renderLink('u:http://example.com|r:4|B:yellow');
//        $this->assertEquals('<span >http://example.com</span>', $result);
        $this->assertEquals('http://example.com', $result);

        $result = $link->renderLink('t:Example|r:4|B:yellow');
//        $this->assertEquals('<span ><img alt="picture.gif" src="picture.gif" title="picture.gif" > Example</span>', $result);
        $this->assertEquals('<img alt="Bullet yellow" src="typo3conf/ext/qfq/Resources/Public/icons/bullet-yellow.gif" title="yellow" > Example', $result);

        $result = $link->renderLink('u:http://example.com|t:Example|r:4|B:yellow');
//        $this->assertEquals('<span >http://example.com</span>', $result);
        $this->assertEquals('http://example.com', $result);

        // r: 5
        $result = $link->renderLink('u:http://example.com|r:5|B:yellow');
        $this->assertEquals('', $result);

        $result = $link->renderLink('t:Example|r:5|B:yellow');
        $this->assertEquals('', $result);

        $result = $link->renderLink('u:http://example.com|t:Example|r:5|B:yellow');
        $this->assertEquals('', $result);
    }

    /**
     * @throws UserReportException
     */
    public function testIcons() {
        $link = new Link($this->sip, DB_INDEX_DEFAULT, true);

        $result = $link->renderLink('u:http://example.com|E');
        $this->assertEquals('<a href="http://example.com" class="btn btn-default" title="Edit" ><span class="glyphicon glyphicon-pencil" ></span></a>', $result);

        $result = $link->renderLink('u:http://example.com|N');
        $this->assertEquals('<a href="http://example.com" class="btn btn-default" title="New" ><span class="glyphicon glyphicon-plus" ></span></a>', $result);

        $result = $link->renderLink('u:http://example.com|D');
        $this->assertEquals('<a href="http://example.com" class="btn btn-default" title="Delete" ><span class="glyphicon glyphicon-trash" ></span></a>', $result);

        $result = $link->renderLink('u:http://example.com|H');
        $this->assertEquals('<a href="http://example.com" class="btn btn-default" title="Help" ><span class="glyphicon glyphicon-question-sign" ></span></a>', $result);

        $result = $link->renderLink('u:http://example.com|I');
        $this->assertEquals('<a href="http://example.com" class="btn btn-default" title="Information" ><span class="glyphicon glyphicon-info-sign" ></span></a>', $result);

        $result = $link->renderLink('u:http://example.com|S');
        $this->assertEquals('<a href="http://example.com" class="btn btn-default" title="Details" ><span class="glyphicon glyphicon-search" ></span></a>', $result);

        $result = $link->renderLink('u:http://example.com|E|o:specific');
        $this->assertEquals('<a href="http://example.com" class="btn btn-default" title="specific" ><span class="glyphicon glyphicon-pencil" ></span></a>', $result);
    }

    /**
     * @throws UserReportException
     */
    public function testBullet() {
        $link = new Link($this->sip, DB_INDEX_DEFAULT, true);

        $result = $link->renderLink('u:http://example.com|B');
        $this->assertEquals('<a href="http://example.com" ><img alt="Bullet green" src="typo3conf/ext/qfq/Resources/Public/icons/bullet-green.gif" title="green" ></a>', $result);

        $result = $link->renderLink('u:http://example.com|B:green');
        $this->assertEquals('<a href="http://example.com" ><img alt="Bullet green" src="typo3conf/ext/qfq/Resources/Public/icons/bullet-green.gif" title="green" ></a>', $result);

        $result = $link->renderLink('u:http://example.com|B:blue');
        $this->assertEquals('<a href="http://example.com" ><img alt="Bullet blue" src="typo3conf/ext/qfq/Resources/Public/icons/bullet-blue.gif" title="blue" ></a>', $result);

        $result = $link->renderLink('u:http://example.com|B:gray');
        $this->assertEquals('<a href="http://example.com" ><img alt="Bullet gray" src="typo3conf/ext/qfq/Resources/Public/icons/bullet-gray.gif" title="gray" ></a>', $result);

        $result = $link->renderLink('u:http://example.com|B:pink');
        $this->assertEquals('<a href="http://example.com" ><img alt="Bullet pink" src="typo3conf/ext/qfq/Resources/Public/icons/bullet-pink.gif" title="pink" ></a>', $result);

        $result = $link->renderLink('u:http://example.com|B:red');
        $this->assertEquals('<a href="http://example.com" ><img alt="Bullet red" src="typo3conf/ext/qfq/Resources/Public/icons/bullet-red.gif" title="red" ></a>', $result);

        $result = $link->renderLink('u:http://example.com|B:yellow');
        $this->assertEquals('<a href="http://example.com" ><img alt="Bullet yellow" src="typo3conf/ext/qfq/Resources/Public/icons/bullet-yellow.gif" title="yellow" ></a>', $result);

        $result = $link->renderLink('u:http://example.com|B|o:specific');
        $this->assertEquals('<a href="http://example.com" title="specific" ><img alt="Bullet green" src="typo3conf/ext/qfq/Resources/Public/icons/bullet-green.gif" title="specific" ></a>', $result);
    }

    /**
     * @throws UserReportException
     */
    public function testChecked() {
        $link = new Link($this->sip, DB_INDEX_DEFAULT, true);

        $result = $link->renderLink('u:http://example.com|C');
        $this->assertEquals('<a href="http://example.com" ><img alt="Checked green" src="typo3conf/ext/qfq/Resources/Public/icons/checked-green.gif" title="green" ></a>', $result);

        $result = $link->renderLink('u:http://example.com|C:green');
        $this->assertEquals('<a href="http://example.com" ><img alt="Checked green" src="typo3conf/ext/qfq/Resources/Public/icons/checked-green.gif" title="green" ></a>', $result);

        $result = $link->renderLink('u:http://example.com|C:blue');
        $this->assertEquals('<a href="http://example.com" ><img alt="Checked blue" src="typo3conf/ext/qfq/Resources/Public/icons/checked-blue.gif" title="blue" ></a>', $result);

        $result = $link->renderLink('u:http://example.com|C:gray');
        $this->assertEquals('<a href="http://example.com" ><img alt="Checked gray" src="typo3conf/ext/qfq/Resources/Public/icons/checked-gray.gif" title="gray" ></a>', $result);

        $result = $link->renderLink('u:http://example.com|C:pink');
        $this->assertEquals('<a href="http://example.com" ><img alt="Checked pink" src="typo3conf/ext/qfq/Resources/Public/icons/checked-pink.gif" title="pink" ></a>', $result);

        $result = $link->renderLink('u:http://example.com|C:red');
        $this->assertEquals('<a href="http://example.com" ><img alt="Checked red" src="typo3conf/ext/qfq/Resources/Public/icons/checked-red.gif" title="red" ></a>', $result);

        $result = $link->renderLink('u:http://example.com|C:yellow');
        $this->assertEquals('<a href="http://example.com" ><img alt="Checked yellow" src="typo3conf/ext/qfq/Resources/Public/icons/checked-yellow.gif" title="yellow" ></a>', $result);

        $result = $link->renderLink('u:http://example.com|C|o:specific');
        $this->assertEquals('<a href="http://example.com" title="specific" ><img alt="Checked green" src="typo3conf/ext/qfq/Resources/Public/icons/checked-green.gif" title="specific" ></a>', $result);
    }

    /**
     * @throws UserReportException
     */
    public function testGlyph() {
        $link = new Link($this->sip, DB_INDEX_DEFAULT, true);

        $result = $link->renderLink('u:http://example.com|G:glyphicon-envelope');
        $this->assertEquals('<a href="http://example.com" class="btn btn-default" title="Details" ><span class="glyphicon glyphicon-envelope" ></span></a>', $result);
    }

    /**
     * @expectedException \qfq\UserReportException
     */
    public function testGlyphException() {
        $link = new Link($this->sip, DB_INDEX_DEFAULT, true);

        // Missing 'glyph'-name
        $result = $link->renderLink('u:http://example.com|G');
    }

    /**
     * @expectedException \qfq\UserReportException
     */
    public function testPictureException1() {
        $link = new Link($this->sip, DB_INDEX_DEFAULT, true);

        // r: default (0)
        $link->renderLink('u:http://www.example.com|P:picture.gif|B');
    }

    /**
     * @expectedException \qfq\UserReportException
     */
    public function testPictureException2() {
        $link = new Link($this->sip, DB_INDEX_DEFAULT, true);

        // r: default (0)
        $link->renderLink('u:http://www.example.com|C|B');
    }

    /**
     * @expectedException \qfq\UserReportException
     */
    public function testPictureException3() {
        $link = new Link($this->sip, DB_INDEX_DEFAULT, true);

        // r: default (0)
        $link->renderLink('u:http://www.example.com|B:green|B:red');
    }

    /**
     * @expectedException \qfq\UserReportException
     */
    public function testPictureException4() {
        $link = new Link($this->sip, DB_INDEX_DEFAULT, true);

        // r: default (0)
        $link->renderLink('u:http://www.example.com|C:green|C:red');
    }

    /**
     * @expectedException \qfq\UserReportException
     */
    public function testGlyphExceptionDouble1() {
        $link = new Link($this->sip, DB_INDEX_DEFAULT, true);

        // r: default (0)
        $link->renderLink('u:http://www.example.com|E|N');
    }

    /**
     * @expectedException \qfq\UserReportException
     */
    public function testGlyphExceptionDouble2() {
        $link = new Link($this->sip, DB_INDEX_DEFAULT, true);

        // r: default (0)
        $link->renderLink('u:http://www.example.com|E|D');
    }

    /**
     * @expectedException \qfq\UserReportException
     */
    public function testGlyphExceptionDouble3() {
        $link = new Link($this->sip, DB_INDEX_DEFAULT, true);

        // r: default (0)
        $link->renderLink('u:http://www.example.com|E|H');
    }

    /**
     * @expectedException \qfq\UserReportException
     */
    public function testGlyphExceptionDouble4() {
        $link = new Link($this->sip, DB_INDEX_DEFAULT, true);

        // r: default (0)
        $link->renderLink('u:http://www.example.com|E|I');
    }

    /**
     * @expectedException \qfq\UserReportException
     */
    public function testGlyphExceptionDouble5() {
        $link = new Link($this->sip, DB_INDEX_DEFAULT, true);

        // r: default (0)
        $link->renderLink('u:http://www.example.com|E|S');
    }

    /**
     * @expectedException \qfq\UserReportException
     */
    public function testGlyphExceptionDouble6() {
        $link = new Link($this->sip, DB_INDEX_DEFAULT, true);

        // r: default (0)
        $link->renderLink('u:http://www.example.com|E|G');
    }

    /**
     */
    public function testLinkUrlParam() {
        $link = new Link($this->sip, DB_INDEX_DEFAULT, true);

        $result = $link->renderLink('u:http://example.com|U:');
        $this->assertEquals('<a href="http://example.com" >http://example.com</a>', $result);

        $result = $link->renderLink('u:http://example.com|U:');
        $this->assertEquals('<a href="http://example.com" >http://example.com</a>', $result);

        $result = $link->renderLink('u:http://example.com|U:a=1234');
        $this->assertEquals('<a href="http://example.com?a=1234" >http://example.com?a=1234</a>', $result);

        $result = $link->renderLink('u:http://example.com|U:a=1234&b=abcd');
        $this->assertEquals('<a href="http://example.com?a=1234&b=abcd" >http://example.com?a=1234&b=abcd</a>', $result);

        $result = $link->renderLink('u:http://example.com|U:a');
        $this->assertEquals('<a href="http://example.com?a" >http://example.com?a</a>', $result);

        $result = $link->renderLink('u:http://example.com|U:a=');
        $this->assertEquals('<a href="http://example.com?a=" >http://example.com?a=</a>', $result);

        $result = $link->renderLink('u:http://example.com?A=hello|U:a=world');
        $this->assertEquals('<a href="http://example.com?A=hello&a=world" >http://example.com?A=hello&a=world</a>', $result);

        $result = $link->renderLink('u:http://example.com?A=hello&B=nice|U:a=world');
        $this->assertEquals('<a href="http://example.com?A=hello&B=nice&a=world" >http://example.com?A=hello&B=nice&a=world</a>', $result);

        $result = $link->renderLink('p:form|U:a=1234');
        $this->assertEquals('<a href="?id=form&a=1234" >?id=form&a=1234</a>', $result);

        $result = $link->renderLink('p:form|U:a=1234&b=abcd');
        $this->assertEquals('<a href="?id=form&a=1234&b=abcd" >?id=form&a=1234&b=abcd</a>', $result);

        $result = $link->renderLink('p:form|U:a');
        $this->assertEquals('<a href="?id=form&a" >?id=form&a</a>', $result);

        $result = $link->renderLink('p:form|U:a=');
        $this->assertEquals('<a href="?id=form&a=" >?id=form&a=</a>', $result);
    }

    /**
     */
    public function testTooltip() {
        $link = new Link($this->sip, DB_INDEX_DEFAULT, true);

        // standard case
        $result = $link->renderLink('u:http://example.com|o:hello world');
        $this->assertEquals('<a href="http://example.com" title="hello world" >http://example.com</a>', $result);

        // standard case, swapped parameter
        $result = $link->renderLink('o:hello world|u:http://example.com');
        $this->assertEquals('<a href="http://example.com" title="hello world" >http://example.com</a>', $result);

        // no text: this is ok
        $result = $link->renderLink('u:http://example.com|o');
        $this->assertEquals('<a href="http://example.com" >http://example.com</a>', $result);

        // no text: this is ok
        $result = $link->renderLink('u:http://example.com|o:');
        $this->assertEquals('<a href="http://example.com" >http://example.com</a>', $result);

        // some text, with double ticks inside
        $result = $link->renderLink('u:http://example.com|o:hello world "some more text" end');
        $this->assertEquals('<a href="http://example.com" title="hello world &quot;some more text&quot; end" >http://example.com</a>', $result);

        // some text with single ticks
        $result = $link->renderLink('u:http://example.com|o:hello world \'some more text\' end');
        $this->assertEquals('<a href="http://example.com" title="hello world \'some more text\' end" >http://example.com</a>', $result);
    }

    /**
     */
    public function testAltText() {
        $link = new Link($this->sip, DB_INDEX_DEFAULT, true);

        // standard case
        $result = $link->renderLink('u:http://example.com|a:hello world|P:image.gif');
        $this->assertEquals('<a href="http://example.com" ><img alt="hello world" src="image.gif" title="image.gif" ></a>', $result);

        // standard: swapped parameter
        //TODO: fixme
//        $result = $link->renderLink('P:image.gif|a:hello world|u:http://example.com');
//        $this->assertEquals('<a href="http://example.com" ><img alt="hello world" src="image.gif" title="image.gif" ></a>', $result);

        // alt text empty
        $result = $link->renderLink('u:http://example.com|a:|P:image.gif');
        $this->assertEquals('<a href="http://example.com" ><img alt="image.gif" src="image.gif" title="image.gif" ></a>', $result);

    }

    /**
     */
    public function testClass() {
        $link = new Link($this->sip, DB_INDEX_DEFAULT, true);

        // no class
        $result = $link->renderLink('u:http://example.com|c:n');
        $this->assertEquals('<a href="http://example.com" >http://example.com</a>', $result);

        // specific class
        $result = $link->renderLink('u:http://example.com|c:myClass');
        $this->assertEquals('<a href="http://example.com" class="myClass" >http://example.com</a>', $result);

        $result = $link->renderLink('u:http://example.com');
        $this->assertEquals('<a href="http://example.com" >http://example.com</a>', $result);

        $result = $link->renderLink('u:http://example.com|c');
        $this->assertEquals('<a href="http://example.com" >http://example.com</a>', $result);

        $result = $link->renderLink('u:http://example.com|c:');
        $this->assertEquals('<a href="http://example.com" >http://example.com</a>', $result);

        $result = $link->renderLink('p:form');
        $this->assertEquals('<a href="?id=form" >?id=form</a>', $result);
    }

    /**
     */
    public function testTarget() {
        $link = new Link($this->sip, DB_INDEX_DEFAULT, true);

        // no target
        $result = $link->renderLink('u:http://example.com|g');
        $this->assertEquals('<a href="http://example.com" >http://example.com</a>', $result);

        // no target
        $result = $link->renderLink('u:http://example.com|g:');
        $this->assertEquals('<a href="http://example.com" >http://example.com</a>', $result);

        // target _blank
        $result = $link->renderLink('u:http://example.com|g:_blank');
        $this->assertEquals('<a href="http://example.com" target="_blank" >http://example.com</a>', $result);

        // target someName
        $result = $link->renderLink('u:http://example.com|g:someName');
        $this->assertEquals('<a href="http://example.com" target="someName" >http://example.com</a>', $result);

        // target someName, swapped parameter
        $result = $link->renderLink('g:someName|u:http://example.com');
        $this->assertEquals('<a href="http://example.com" target="someName" >http://example.com</a>', $result);
    }

    /**
     */
    public function testRight() {
        $link = new Link($this->sip, DB_INDEX_DEFAULT, true);

        // Bullet, LEFT (Standard)
        $result = $link->renderLink('u:http://example.com|t:Hello World|B');
        $this->assertEquals('<a href="http://example.com" ><img alt="Bullet green" src="typo3conf/ext/qfq/Resources/Public/icons/bullet-green.gif" title="green" > Hello World</a>', $result);

        // Bullet, RIGHT
        $result = $link->renderLink('u:http://example.com|t:Hello World|B|R');
        $this->assertEquals('<a href="http://example.com" >Hello World <img alt="Bullet green" src="typo3conf/ext/qfq/Resources/Public/icons/bullet-green.gif" title="green" ></a>', $result);

        // Checked, LEFT (Standard)
        $result = $link->renderLink('u:http://example.com|t:Hello World|C');
        $this->assertEquals('<a href="http://example.com" ><img alt="Checked green" src="typo3conf/ext/qfq/Resources/Public/icons/checked-green.gif" title="green" > Hello World</a>', $result);

        // Checked, RIGHT
        $result = $link->renderLink('u:http://example.com|t:Hello World|C|R');
        $this->assertEquals('<a href="http://example.com" >Hello World <img alt="Checked green" src="typo3conf/ext/qfq/Resources/Public/icons/checked-green.gif" title="green" ></a>', $result);

        // Picture, LEFT (Standard)
        $result = $link->renderLink('u:http://example.com|t:Hello World|P:image.gif');
        $this->assertEquals('<a href="http://example.com" ><img alt="image.gif" src="image.gif" title="image.gif" > Hello World</a>', $result);

        // Picture, RIGHT
        $result = $link->renderLink('u:http://example.com|t:Hello World|P:image.gif|R');
        $this->assertEquals('<a href="http://example.com" >Hello World <img alt="image.gif" src="image.gif" title="image.gif" ></a>', $result);

        // swapped param
        $result = $link->renderLink('R|P:image.gif|t:Hello World|u:http://example.com');
        $this->assertEquals('<a href="http://example.com" >Hello World <img alt="image.gif" src="image.gif" title="image.gif" ></a>', $result);
    }


    /**
     */
    public function testSip() {
        $link = new Link($this->sip, DB_INDEX_DEFAULT, true);

        // Sip: URL, s
        $result = $link->renderLink('u:?form&r=12&xId=2345&L=1&type=99&gId=55|s');
        $this->assertEquals('<a href="index.php?id=form&L=1&type=99&s=badcaffee1234" >index.php?id=form&L=1&type=99&s=badcaffee1234</a>', $result);

        // Sip: URL, s:0
        $result = $link->renderLink('u:?form&r=12&xId=2345&L=1&type=99&gId=55|s:0');
        $this->assertEquals('<a href="index.php?form&r=12&xId=2345&L=1&type=99&gId=55" >index.php?form&r=12&xId=2345&L=1&type=99&gId=55</a>', $result);

        // Sip: URL, s:1
        $result = $link->renderLink('u:?form&r=12&xId=2345&L=1&type=99&gId=55|s:1');
        $this->assertEquals('<a href="index.php?id=form&L=1&type=99&s=badcaffee1234" >index.php?id=form&L=1&type=99&s=badcaffee1234</a>', $result);

        // Sip: Page, s
        $result = $link->renderLink('p:form&r=12&xId=2345&L=1&type=99&gId=55|s');
        $this->assertEquals('<a href="index.php?id=form&L=1&type=99&s=badcaffee1234" >index.php?id=form&L=1&type=99&s=badcaffee1234</a>', $result);

        // Sip: Page, s:0
        $result = $link->renderLink('p:form&r=12&xId=2345&L=1&type=99&gId=55|s:0');
        $this->assertEquals('<a href="?id=form&r=12&xId=2345&L=1&type=99&gId=55" >?id=form&r=12&xId=2345&L=1&type=99&gId=55</a>', $result);

        // Sip: Page, s:1
        $result = $link->renderLink('p:form&r=12&xId=2345&L=1&type=99&gId=55|s:1');
        $this->assertEquals('<a href="index.php?id=form&L=1&type=99&s=badcaffee1234" >index.php?id=form&L=1&type=99&s=badcaffee1234</a>', $result);
    }

    /**
     * @expectedException \qfq\UserReportException
     */
    public function testSipException1() {
        $link = new Link($this->sip, DB_INDEX_DEFAULT, true);

        // r: default (0)
        $link->renderLink('u:http://www.example.com|s:s');
    }

    /**
     * @expectedException \qfq\UserReportException
     */
    public function testSipException2() {
        $link = new Link($this->sip, DB_INDEX_DEFAULT, true);

        // r: default (0)
        $link->renderLink('u:http://www.example.com|s:2');
    }

    /**
     */
    public function testQuestion() {
        $link = new Link($this->sip, DB_INDEX_DEFAULT, true);

        $js = <<<EOF
id="12345" onClick="var alert = new QfqNS.Alert({ message: 'Please confirm', type: 'info', modal: true, timeout: 0, buttons: [
    { label: 'Ok', eventName: 'ok' },
    { label: 'Cancel',eventName: 'cancel'}
] } );
alert.on('alert.ok', function() {
  window.location = $('#12345').attr('href');
});

alert.show();
return false;"
EOF;

        // Question: all default
        $result = $link->renderLink('p:person|c:n|q');
        $this->assertEquals('<a href="?id=person" ' . $js . ' >?id=person</a>', $result);

        // Question: all default
        $result = $link->renderLink('p:person|c:n|q:');
        $this->assertEquals('<a href="?id=person" ' . $js . ' >?id=person</a>', $result);

        // Question: individual text
        $js = str_replace('Please confirm', 'do you really want', $js);
        $result = $link->renderLink('p:person|c:n|q:do you really want');
        $this->assertEquals('<a href="?id=person" ' . $js . ' >?id=person</a>', $result);

        // Question: individual text, level: warning
        $js = str_replace('info', 'warning', $js);
        $result = $link->renderLink('p:person|c:n|q:do you really want:warning');
        $this->assertEquals('<a href="?id=person" ' . $js . ' >?id=person</a>', $result);

        // Question: individual text, level: warning, positive button: I do
        $js = str_replace('Ok', 'I do', $js);
        $result = $link->renderLink('p:person|c:n|q:do you really want:warning:I do');
        $this->assertEquals('<a href="?id=person" ' . $js . ' >?id=person</a>', $result);

        // Question: individual text, level: warning, positive button: I do, negative button: Shut up
        $js = str_replace('Cancel', 'Shut up', $js);
        $result = $link->renderLink('p:person|c:n|q:do you really want:warning:I do:Shut up');
        $this->assertEquals('<a href="?id=person" ' . $js . ' >?id=person</a>', $result);

        // Question: individual text (with escaped colon)), level: warning, positive button: I do, negative button: Shut up
        $js = str_replace('do you really want', 'My Question:some nice value', $js);
        $result = $link->renderLink("p:person|c:n|q:My Question\\:some nice value:warning:I do:Shut up");
        $this->assertEquals('<a href="?id=person" ' . $js . ' >?id=person</a>', $result);

        // Question: individual text (with escaped colon)), level: warning, positive button: I do (with escaped colon), negative button: Shut up
        $js = str_replace('I do', 'I do: hurry up', $js);
        $result = $link->renderLink("p:person|c:n|q:My Question\\:some nice value:warning:I do\\: hurry up:Shut up");
        $this->assertEquals('<a href="?id=person" ' . $js . ' >?id=person</a>', $result);

        // Timeout:
        $js = str_replace('timeout: 0', 'timeout: 10000', $js);
        $result = $link->renderLink("p:person|c:n|q:My Question\\:some nice value:warning:I do\\: hurry up:Shut up:10");
        $this->assertEquals('<a href="?id=person" ' . $js . ' >?id=person</a>', $result);

        // Modal: 1
        $result = $link->renderLink("p:person|c:n|q:My Question\\:some nice value:warning:I do\\: hurry up:Shut up:10:1");
        $this->assertEquals('<a href="?id=person" ' . $js . ' >?id=person</a>', $result);

        // Modal: 0
        $js = str_replace('modal: true', 'modal: false', $js);
        $result = $link->renderLink("p:person|c:n|q:My Question\\:some nice value:warning:I do\\: hurry up:Shut up:10:0");
        $this->assertEquals('<a href="?id=person" ' . $js . ' >?id=person</a>', $result);

    }

    /**
     *
     */
    public function testDelete() {
        $link = new Link($this->sip, DB_INDEX_DEFAULT, true);

        // Report Delete action, default: 'Report', no Icon
        $result = $link->renderLink('U:form=Person&r=123|x');
        $this->assertEquals('<a href="typo3conf/ext/qfq/qfq/api/delete.php?s=badcaffee1234" >typo3conf/ext/qfq/qfq/api/delete.php?s=badcaffee1234</a>', $result);
        // Check das via '_paged' SIP_MODE_ANSWER and SIP_TARGET_URL has been set.
        $result = \qfq\Session::get('badcaffee1234');
        $this->assertEquals('__dbIndexData=1&_modeAnswer=html&_targetUrl=localhost&form=Person&r=123', $result);

        // Report Delete action, explicit 'Report', no Icon
        $result = $link->renderLink('U:form=PersonA&r=1234|x:r');
        $this->assertEquals('<a href="typo3conf/ext/qfq/qfq/api/delete.php?s=badcaffee1234" >typo3conf/ext/qfq/qfq/api/delete.php?s=badcaffee1234</a>', $result);
        // Check das via '_paged' SIP_MODE_ANSWER and SIP_TARGET_URL has been set.
        $result = \qfq\Session::get('badcaffee1234');
        $this->assertEquals('__dbIndexData=1&_modeAnswer=html&_targetUrl=localhost&form=PersonA&r=1234', $result);

        // Report Delete action, explicit 'Report', Text
        $result = $link->renderLink('U:form=PersonAa&r=2234|x:r|t:Delete Record');
        $this->assertEquals('<a href="typo3conf/ext/qfq/qfq/api/delete.php?s=badcaffee1234" >Delete Record</a>', $result);
        // Check das via '_paged' SIP_MODE_ANSWER and SIP_TARGET_URL has been set.
        $result = \qfq\Session::get('badcaffee1234');
        $this->assertEquals('__dbIndexData=1&_modeAnswer=html&_targetUrl=localhost&form=PersonAa&r=2234', $result);

        // Report Delete action, with Icon
        $result = $link->renderLink('U:form=PersonB&r=1235|x|D');
        $this->assertEquals('<a href="typo3conf/ext/qfq/qfq/api/delete.php?s=badcaffee1234" class="btn btn-default" title="Delete" ><span class="glyphicon glyphicon-trash" ></span></a>', $result);
        // Check das via '_paged' SIP_MODE_ANSWER and SIP_TARGET_URL has been set.
        $result = \qfq\Session::get('badcaffee1234');
        $this->assertEquals('__dbIndexData=1&_modeAnswer=html&_targetUrl=localhost&form=PersonB&r=1235', $result);

        // Report Delete action: explicit php file, default: 'Report', no Icon
        $result = $link->renderLink('u:mydelete.php|U:form=PersonC&r=1236|x');
        $this->assertEquals('<a href="mydelete.php?s=badcaffee1234" >mydelete.php?s=badcaffee1234</a>', $result);
        // Check das via '_paged' SIP_MODE_ANSWER and SIP_TARGET_URL has been set.
        $result = \qfq\Session::get('badcaffee1234');
        $this->assertEquals('__dbIndexData=1&_modeAnswer=html&_targetUrl=localhost&form=PersonC&r=1236', $result);
    }

    /**
     * @expectedException \qfq\UserReportException
     */
    public function testDeleteException1() {
        $link = new Link($this->sip, DB_INDEX_DEFAULT, true);

        // Missing recordId
        $link->renderLink('U:form=Person&r=|x');
    }


    /**
     *
     */
    protected function setUp() {
        parent::setUp();

        $this->store = Store::getInstance('', true);
        $this->sip = new Sip('fakesessionname', true);
        $this->sip->sipUniqId('badcaffee1234');

    }
}
